package com.muzhi.tianhe.controller;

import com.muzhi.tianhe.entity.Tianhe;
import com.muzhi.tianhe.entity.vo.TianheDataVo;
import com.muzhi.tianhe.entity.vo.TianheQuery;
import com.muzhi.tianhe.exceptionhander.R;
import com.muzhi.tianhe.service.TbService;
import com.muzhi.tianhe.util.UtilExcel;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.http.HttpResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

/**
 * @Author ZhangKan
 * @Data 15:03
 */

@Api(description = "TB接口")
@EnableScheduling
@CrossOrigin
@RestController
@RequestMapping("/tbservice")
public class TbController {

    @Autowired
    private TbService tbService;

    //获取项目信息
    @Scheduled(cron = "0 0 0,12 * * ? ")
    @ApiOperation(value = "获取项目信息")
    @PostMapping("getXiangmu")
    public String getXiangmu(){
        tbService.asyncXm();
        return "success";
    }

    @GetMapping("syncProject")
    public String syncProject(String id){
        tbService.asyncXm(id);
        return "success";
    }

    //查询数据库中tianhe表的所有数据
    @ApiOperation(value = "查询数据库中tianhe表的所有数据")
    @GetMapping("getTianhe")
    public List<Tianhe> getTianhe(){
        List<Tianhe> tianheList = tbService.getTianhe();
        return tianheList;
    }

    //删除tianhe表中的所有数据
    @ApiOperation(value = "删除tianhe表中的所有数据")
    @PostMapping("removeTianhe")
    public R removeTianhe(){
        String result = tbService.removeTianhe();
        return R.ok().data("result", result);
    }

    //获取数据库中tianhe实体类表数据并放入vo对象进行前端接口调用展示数据
    @ApiOperation(value = "获取天合TB项目信息")
    @GetMapping("getTianheTB")
    public TianheDataVo getTianheTB(Integer page, Integer limit,
                                    TianheQuery tianheQuery){

//        @RequestBody是使用json传递数据，把json数据封装到对应的对象里面，注意：需要用Post提交方式，Get取不到值
//        (required = false)表示参数值可以为空
//        @ApiParam(name = "teacherQuery", value = "自定义讲师查询条件", required = true)
//        @RequestBody(required = false) EmployeeQuery employeeVoQuery

        TianheDataVo tianheTB = tbService.getTianheTB(page,limit,tianheQuery);
        return tianheTB;
    }

    //获取数据库中tianhe实体类表数据并导出
    @ApiOperation(value = "导出天合TB项目信息")
    @GetMapping("exportTianheTB")
    public void exportTianheTB(HttpServletResponse response,TianheQuery tianheQuery) throws IOException {
        TianheDataVo tianheTB = tbService.getTianheTB(1,Integer.MAX_VALUE,tianheQuery);
        UtilExcel.exportListByTemplate(response,tianheTB.getData(),Tianhe.class,"周报");
    }
}
