package com.muzhi.tianhe.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceResponse;
import com.aliyun.tea.TeaException;
import com.aliyun.teautil.models.RuntimeOptions;
import com.muzhi.tianhe.service.AccessTokenService;
import com.muzhi.tianhe.service.DingService;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.net.URLDecoder;
import java.util.HashMap;
import java.util.Map;


/**
 * @Author ZhangKan
 * @Data 15:39
 */

@Service
@Slf4j
public class DingServiceImpl implements DingService {

    @Autowired
    private AccessTokenService accessTokenService;

    //易控TB企业ID（tbOrganizationId）
    public static String tbOrganizationId = "61db91aa9404856a3150c52a";

    //钉钉接口服务【审批单实例详情】
    public static com.aliyun.dingtalkworkflow_1_0.Client createClient() throws Exception {
        com.aliyun.teaopenapi.models.Config config = new com.aliyun.teaopenapi.models.Config();
        config.protocol = "https";
        config.regionId = "central";
        return new com.aliyun.dingtalkworkflow_1_0.Client(config);
    }

    //获取钉钉OA审批信息（任务）
    @Override
    public String getDingOA(String processInstances) {

//        processInstances = "kYMesPtPS1CkGVVDuePG3Q01951689761375";

        log.info("钉钉连接器传参审批实例ID：" + processInstances);

        //根据审批实力id获取审批单详情，并获取评论添加的tb任务详情，最后取值任务id
        //一、获取【任务完成审批】详情
        JSONObject jsonObject = null;
        try {
            com.aliyun.dingtalkworkflow_1_0.Client client = createClient();
            com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceHeaders getProcessInstanceHeaders = new com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceHeaders();
            getProcessInstanceHeaders.xAcsDingtalkAccessToken = accessTokenService.getAccessToken();
            com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceRequest getProcessInstanceRequest = new com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceRequest()
                    //审批实例id
                    .setProcessInstanceId(processInstances);

            GetProcessInstanceResponse processInstanceWithOptions = client.getProcessInstanceWithOptions(getProcessInstanceRequest, getProcessInstanceHeaders, new RuntimeOptions());
//            System.out.println(JSON.toJSONString(processInstanceWithOptions.getBody()));
            String s = JSON.toJSONString(processInstanceWithOptions.getBody());
            jsonObject = JSON.parseObject(s);
            System.out.println("审批详情接口object对象：" + jsonObject);
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                // err 中含有 code 和 message 属性，可帮助开发定位问题
            }
        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                // err 中含有 code 和 message 属性，可帮助开发定位问题
            }
        }
        JSONObject result = jsonObject.getJSONObject("result");
        System.out.println("result：" + result);
        JSONArray formComponentValues = result.getJSONArray("formComponentValues");
        System.out.println("formComponentValues：" + formComponentValues);

        JSONArray operationRecords = result.getJSONArray("operationRecords");
        JSONObject jsonObject1 = operationRecords.getJSONObject(1);
        String remark = jsonObject1.getString("remark");
        System.out.println("评论地址：" + remark);

        String substring = remark.substring(remark.indexOf("(") + 1, remark.indexOf(")"));
        System.out.println("截取出来的url：" + substring);


        //审批状态
        //NEW：新创建
        //RUNNING：审批中
        //TERMINATED：被终止
        //COMPLETED：完成
        //CANCELED：取消
        String status = result.getString("status");
        System.out.println("当前审批单状态：" + status);

        String shenpijieguo = null;
        try {
            shenpijieguo = result.getString("result");
            System.out.println("当前审批单结果：" + shenpijieguo);
        }catch (Exception e){
            e.printStackTrace();
        }
        if (status.equals("COMPLETED") && shenpijieguo.equals("agree")) {
            log.info("审批通过，开始执行更新TB任务状态动作～");

            String url = substring;
            for (int i = 0; i < 3; i++) {


                String decodedUrl = null;
                try {
                    decodedUrl = URLDecoder.decode(url, "UTF-8");
                } catch (Exception e) {
                    e.printStackTrace();
                }
                System.out.println("第" + i + "次解码后：" + decodedUrl);

                url = decodedUrl;
            }
            System.out.println("3次最终解码后的url：" + url);

            //截取指定的task任务编号
//            String task = url.substring(url.indexOf("tasks/") + 6, url.indexOf("&productCode"));
            //TODO 特殊处理
            String task = url.substring(url.indexOf("tasks/") + 6, url.indexOf("?isDingtalkHeaderHidden"));
            System.out.println("截取的task任务编号：" + task);

            //查询任务详情，获取该任务的项目ID
            OkHttpClient client22 = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType22 = MediaType.parse("text/plain");
            RequestBody body22 = RequestBody.create(mediaType22, "");
            Request request22 = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/task/query?taskId=" + task)
//                .method("GET", body)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .build();

            int code22 = 0;
            JSONArray result22 = null;
            try {
                Response response = client22.newCall(request22).execute();
                String bodyStr = response.body().string();
                JSONObject jsonObject22 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject22);
                code22 = jsonObject22.getInteger("code");

                result22 = jsonObject22.getJSONArray("result");
            } catch (IOException e) {
                e.printStackTrace();
            }
            //项目id
            String projectId = result22.getJSONObject(0).getString("projectId");
            //创建人id
            String id1 = result22.getJSONObject(0).getString("creatorId");
            System.out.println("查询【任务详情】接口返回的状态码（200成功，400异常）：" + code22);
            System.out.println("项目ID：" + projectId);
            System.out.println("创建人id：" + id1);

            //查询项目的工作流（获取默认工作流ID）
            OkHttpClient client33 = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType33 = MediaType.parse("text/plain");
            RequestBody body33 = RequestBody.create(mediaType33, "");
            Request request33 = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/project/" + projectId + "/taskflow/search")
//                .method("GET", body33)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .build();
            int code33 = 0;
            JSONArray result33 = null;
            try {
                Response response = client33.newCall(request33).execute();
                String bodyStr = response.body().string();
                JSONObject jsonObject33 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject33);
                code33 = jsonObject33.getInteger("code");

                result33 = jsonObject33.getJSONArray("result");
            } catch (IOException e) {
                e.printStackTrace();
            }

            String gongzuoliuID = null;
            for (int i = 0; i < result33.size(); i++) {
                String name = result33.getJSONObject(i).getString("name");
                System.out.println("工作流模板名称：" + name);

                if (name.equals("默认工作流")){
                    String id = result33.getJSONObject(i).getString("id");
                    System.out.println("默认工作流id：" + id);
                    gongzuoliuID = id;
                }
            }

//            String gongzuoliuID = result33.getJSONObject(0).getString("id");
            System.out.println("查询【项目的工作流】接口返回的状态码（200成功，400异常）：" + code33);
            System.out.println("项目工作流ID（默认工作流）：" + gongzuoliuID);

            //根据项目默认工作流ID获取项目中任务的各个状态ID
            OkHttpClient client44 = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType44 = MediaType.parse("text/plain");
            RequestBody body44 = RequestBody.create(mediaType44, "");
            Request request44 = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/project/" + projectId + "/taskflowstatus/search?tfIds=" + gongzuoliuID)
//                .method("GET", body44)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .build();
            int code44 = 0;
            JSONArray result44 = null;
            try {
                Response response44 = client44.newCall(request44).execute();
                String bodyStr = response44.body().string();
                JSONObject jsonObject44 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject44);
                code44 = jsonObject44.getInteger("code");

                result44 = jsonObject44.getJSONArray("result");
            } catch (IOException e) {
                e.printStackTrace();
            }

//        String renwuID = result44.getJSONObject(2).getString("id");

            String renwuID = null;
            for (int i = 0; i < result44.size(); i++) {
                String name = result44.getJSONObject(i).getString("name");
                System.out.println("任务状态名称：" + name);
                if (name.equals("已完成")) {
                    String id = result44.getJSONObject(i).getString("id");
                    renwuID = id;
                }
            }

//        result44.stream().filter()

            System.out.println("--------------------------");
            System.out.println("查询【项目默认工作流ID】接口返回的状态码（200成功，400异常）：" + code44);
            System.out.println("项目工作流-->任务完成ID：" + renwuID);

            //TB任务状态id
            // 未处理：64afa1090c87b662c9b28cda
            // 已完成：64afa1090c87b662c9b28cdc

            //柏TBuserid：61c68fc2947cb43084ec0113
            //牧之TBuserid：5f3a250040ce230ba377bf3c
            //更新TB任务状态
            log.info("开始执行更新TB任务状态操作...");

            Map map = new HashMap();
            map.put("taskflowstatusId", renwuID);
            String mapJson = JSON.toJSONString(map);
            System.out.println("查询条件mapJson：" + mapJson);

            OkHttpClient client = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType = MediaType.parse("application/json");
            RequestBody body = RequestBody.create(mediaType, mapJson);
            Request request = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/task/" + task + "/taskflowstatus")
                    .method("PUT", body)
                    //TODO 此处需要替换成项目创建者？？？【已和客户确认】
                    .addHeader("x-operator-id", id1)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .addHeader("Content-Type", "application/json")
                    .build();

            int code11 = 0;
            try {
                Response response = client.newCall(request).execute();
                String bodyStr = response.body().string();
                JSONObject jsonObject11 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject11);
                code11 = jsonObject11.getInteger("code");
            } catch (IOException e) {
                e.printStackTrace();
            }
            System.out.println("更新【TB任务状态】接口返回的状态码（200成功，400异常）：" + code11);
        }

        return "TB任务状态更新完成！";
    }

    //获取钉钉OA审批信息（变更）
    @Override
    public String getDingOAbiangeng(String processInstances,String shenheyijian) {

//        processInstances = "kYMesPtPS1CkGVVDuePG3Q01951689761375";

        log.info("钉钉连接器传参审批实例ID：" + processInstances);
        log.info("审核意见：" + shenheyijian);

        //根据审批实力id获取审批单详情，并获取评论添加的tb任务详情，最后取值任务id
        //一、获取【任务完成审批】详情
        JSONObject jsonObject = null;
        try {
            com.aliyun.dingtalkworkflow_1_0.Client client = createClient();
            com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceHeaders getProcessInstanceHeaders = new com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceHeaders();
            getProcessInstanceHeaders.xAcsDingtalkAccessToken = accessTokenService.getAccessToken();
            com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceRequest getProcessInstanceRequest = new com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceRequest()
                    //审批实例id
                    .setProcessInstanceId(processInstances);

            GetProcessInstanceResponse processInstanceWithOptions = client.getProcessInstanceWithOptions(getProcessInstanceRequest, getProcessInstanceHeaders, new RuntimeOptions());
//            System.out.println(JSON.toJSONString(processInstanceWithOptions.getBody()));
            String s = JSON.toJSONString(processInstanceWithOptions.getBody());
            jsonObject = JSON.parseObject(s);
            System.out.println("审批详情接口object对象：" + jsonObject);
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                // err 中含有 code 和 message 属性，可帮助开发定位问题
            }
        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                // err 中含有 code 和 message 属性，可帮助开发定位问题
            }
        }
        JSONObject result = jsonObject.getJSONObject("result");
        System.out.println("result：" + result);
        JSONArray formComponentValues = result.getJSONArray("formComponentValues");
        System.out.println("formComponentValues：" + formComponentValues);

        JSONArray operationRecords = result.getJSONArray("operationRecords");
        JSONObject jsonObject1 = operationRecords.getJSONObject(1);
        String remark = jsonObject1.getString("remark");
        System.out.println("评论地址：" + remark);

        String substring = remark.substring(remark.indexOf("(") + 1, remark.indexOf(")"));
        System.out.println("截取出来的url：" + substring);


        //审批状态
        //NEW：新创建
        //RUNNING：审批中
        //TERMINATED：被终止
        //COMPLETED：完成
        //CANCELED：取消
        String status = result.getString("status");
        System.out.println("当前审批单状态：" + status);

        String shenpijieguo = null;
        try {
            shenpijieguo = result.getString("result");
            System.out.println("当前审批单结果：" + shenpijieguo);
        }catch (Exception e){
            e.printStackTrace();
        }
        if (status.equals("COMPLETED") && shenpijieguo.equals("agree")) {
            log.info("审批通过，开始执行更新TB任务状态动作～");

            String url = substring;
            for (int i = 0; i < 3; i++) {


                String decodedUrl = null;
                try {
                    decodedUrl = URLDecoder.decode(url, "UTF-8");
                } catch (Exception e) {
                    e.printStackTrace();
                }
                System.out.println("第" + i + "次解码后：" + decodedUrl);

                url = decodedUrl;
            }
            System.out.println("3次最终解码后的url：" + url);

            //截取指定的task任务编号
            String task = url.substring(url.indexOf("tasks/") + 6, url.indexOf("&productCode"));
            System.out.println("截取的task任务编号：" + task);

            //查询任务详情，获取该任务的项目ID
            OkHttpClient client22 = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType22 = MediaType.parse("text/plain");
            RequestBody body22 = RequestBody.create(mediaType22, "");
            Request request22 = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/task/query?taskId=" + task)
//                .method("GET", body)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .build();

            int code22 = 0;
            JSONArray result22 = null;
            try {
                Response response = client22.newCall(request22).execute();
                String bodyStr = response.body().string();
                JSONObject jsonObject22 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject22);
                code22 = jsonObject22.getInteger("code");

                result22 = jsonObject22.getJSONArray("result");
            } catch (IOException e) {
                e.printStackTrace();
            }
            //项目id
            String projectId = result22.getJSONObject(0).getString("projectId");
            //创建人id
            String id1 = result22.getJSONObject(0).getString("creatorId");
            System.out.println("查询【任务详情】接口返回的状态码（200成功，400异常）：" + code22);
            System.out.println("项目ID：" + projectId);
            System.out.println("创建人id：" + id1);

            //查询项目的工作流（获取变更任务工作流ID）
            OkHttpClient client33 = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType33 = MediaType.parse("text/plain");
            RequestBody body33 = RequestBody.create(mediaType33, "");
            Request request33 = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/project/" + projectId + "/taskflow/search")
//                .method("GET", body33)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .build();
            int code33 = 0;
            JSONArray result33 = null;
            try {
                Response response = client33.newCall(request33).execute();
                String bodyStr = response.body().string();
                JSONObject jsonObject33 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject33);
                code33 = jsonObject33.getInteger("code");

                result33 = jsonObject33.getJSONArray("result");
            } catch (IOException e) {
                e.printStackTrace();
            }

            String gongzuoliuID = null;
            for (int i = 0; i < result33.size(); i++) {
                String name = result33.getJSONObject(i).getString("name");
                System.out.println("工作流名称：" + name);
                if (name.equals("变更")){
                    String id = result33.getJSONObject(i).getString("id");
                    System.out.println("工作流id：" + id);
                    gongzuoliuID = id;
                }
            }

//            String gongzuoliuID = result33.getJSONObject(0).getString("id");
            System.out.println("查询【项目的工作流】接口返回的状态码（200成功，400异常）：" + code33);
            System.out.println("项目工作流ID（任务变更工作流）：" + gongzuoliuID);

            //根据项目默认工作流ID获取项目中任务的各个状态ID
            OkHttpClient client44 = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType44 = MediaType.parse("text/plain");
            RequestBody body44 = RequestBody.create(mediaType44, "");
            Request request44 = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/project/" + projectId + "/taskflowstatus/search?tfIds=" + gongzuoliuID)
//                .method("GET", body44)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .build();
            int code44 = 0;
            JSONArray result44 = null;
            try {
                Response response44 = client44.newCall(request44).execute();
                String bodyStr = response44.body().string();
                JSONObject jsonObject44 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject44);
                code44 = jsonObject44.getInteger("code");

                result44 = jsonObject44.getJSONArray("result");
            } catch (IOException e) {
                e.printStackTrace();
            }

//        String renwuID = result44.getJSONObject(2).getString("id");

            String zhuangtai = null;
            if (shenheyijian.equals("同意")){
                zhuangtai = "审核通过";
            }else {
                zhuangtai = "审核不通过";
            }

            String renwuID = null;
            for (int i = 0; i < result44.size(); i++) {
                String name = result44.getJSONObject(i).getString("name");
                System.out.println("任务状态名称：" + name);
                if (name.equals(zhuangtai)) {
                    String id = result44.getJSONObject(i).getString("id");
                    renwuID = id;
                }
            }

//        result44.stream().filter()

            System.out.println("--------------------------");
            System.out.println("查询【项目任务变更工作流ID】接口返回的状态码（200成功，400异常）：" + code44);
            System.out.println("项目工作流-->任务完成ID：" + renwuID);

            //TB任务状态id
            // 未处理：64afa1090c87b662c9b28cda
            // 已完成：64afa1090c87b662c9b28cdc

            //柏TBuserid：61c68fc2947cb43084ec0113
            //牧之TBuserid：5f3a250040ce230ba377bf3c
            //更新TB任务状态
            log.info("开始执行更新TB任务状态操作...");

            Map map = new HashMap();
            map.put("taskflowstatusId", renwuID);
            String mapJson = JSON.toJSONString(map);
            System.out.println("查询条件mapJson：" + mapJson);

            OkHttpClient client = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType = MediaType.parse("application/json");
            RequestBody body = RequestBody.create(mediaType, mapJson);
            Request request = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/task/" + task + "/taskflowstatus")
                    .method("PUT", body)
                    //TODO 此处需要替换成项目创建者？？？【已和客户确认】
                    .addHeader("x-operator-id", id1)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .addHeader("Content-Type", "application/json")
                    .build();

            int code11 = 0;
            try {
                Response response = client.newCall(request).execute();
                String bodyStr = response.body().string();
                JSONObject jsonObject11 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject11);
                code11 = jsonObject11.getInteger("code");
            } catch (IOException e) {
                e.printStackTrace();
            }
            System.out.println("更新【TB任务状态】接口返回的状态码（200成功，400异常）：" + code11);
        }

        return "TB任务【变更】状态更新完成！";
    }

    //获取钉钉OA审批信息（关键任务变更）
    @Override
    public String getDingOAbiangeng2(String processInstances,String shenheyijian) {

//        processInstances = "kYMesPtPS1CkGVVDuePG3Q01951689761375";

        log.info("钉钉连接器传参审批实例ID：" + processInstances);
        log.info("审核意见：" + shenheyijian);

        //根据审批实力id获取审批单详情，并获取评论添加的tb任务详情，最后取值任务id
        //一、获取【任务完成审批】详情
        JSONObject jsonObject = null;
        try {
            com.aliyun.dingtalkworkflow_1_0.Client client = createClient();
            com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceHeaders getProcessInstanceHeaders = new com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceHeaders();
            getProcessInstanceHeaders.xAcsDingtalkAccessToken = accessTokenService.getAccessToken();
            com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceRequest getProcessInstanceRequest = new com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceRequest()
                    //审批实例id
                    .setProcessInstanceId(processInstances);

            GetProcessInstanceResponse processInstanceWithOptions = client.getProcessInstanceWithOptions(getProcessInstanceRequest, getProcessInstanceHeaders, new RuntimeOptions());
//            System.out.println(JSON.toJSONString(processInstanceWithOptions.getBody()));
            String s = JSON.toJSONString(processInstanceWithOptions.getBody());
            jsonObject = JSON.parseObject(s);
            System.out.println("审批详情接口object对象：" + jsonObject);
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                // err 中含有 code 和 message 属性，可帮助开发定位问题
            }
        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                // err 中含有 code 和 message 属性，可帮助开发定位问题
            }
        }
        JSONObject result = jsonObject.getJSONObject("result");
        System.out.println("result：" + result);
        JSONArray formComponentValues = result.getJSONArray("formComponentValues");
        System.out.println("formComponentValues：" + formComponentValues);

        JSONArray operationRecords = result.getJSONArray("operationRecords");
        JSONObject jsonObject1 = operationRecords.getJSONObject(1);
        String remark = jsonObject1.getString("remark");
        System.out.println("评论地址：" + remark);

        String substring = remark.substring(remark.indexOf("(") + 1, remark.indexOf(")"));
        System.out.println("截取出来的url：" + substring);


        //审批状态
        //NEW：新创建
        //RUNNING：审批中
        //TERMINATED：被终止
        //COMPLETED：完成
        //CANCELED：取消
        String status = result.getString("status");
        System.out.println("当前审批单状态：" + status);

        String shenpijieguo = null;
        try {
            shenpijieguo = result.getString("result");
            System.out.println("当前审批单结果：" + shenpijieguo);
        }catch (Exception e){
            e.printStackTrace();
        }
        if (status.equals("COMPLETED") && shenpijieguo.equals("agree")) {
            log.info("审批通过，开始执行更新TB任务状态动作～");

            String url = substring;
            for (int i = 0; i < 3; i++) {


                String decodedUrl = null;
                try {
                    decodedUrl = URLDecoder.decode(url, "UTF-8");
                } catch (Exception e) {
                    e.printStackTrace();
                }
                System.out.println("第" + i + "次解码后：" + decodedUrl);

                url = decodedUrl;
            }
            System.out.println("3次最终解码后的url：" + url);

            //截取指定的task任务编号
            String task = url.substring(url.indexOf("tasks/") + 6, url.indexOf("&productCode"));
            System.out.println("截取的task任务编号：" + task);

            //查询任务详情，获取该任务的项目ID
            OkHttpClient client22 = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType22 = MediaType.parse("text/plain");
            RequestBody body22 = RequestBody.create(mediaType22, "");
            Request request22 = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/task/query?taskId=" + task)
//                .method("GET", body)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .build();

            int code22 = 0;
            JSONArray result22 = null;
            try {
                Response response = client22.newCall(request22).execute();
                String bodyStr = response.body().string();
                JSONObject jsonObject22 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject22);
                code22 = jsonObject22.getInteger("code");

                result22 = jsonObject22.getJSONArray("result");
            } catch (IOException e) {
                e.printStackTrace();
            }
            //项目id
            String projectId = result22.getJSONObject(0).getString("projectId");
            //创建人id
            String id1 = result22.getJSONObject(0).getString("creatorId");
            System.out.println("查询【任务详情】接口返回的状态码（200成功，400异常）：" + code22);
            System.out.println("项目ID：" + projectId);
            System.out.println("创建人id：" + id1);

            //查询项目的工作流（获取变更任务工作流ID）
            OkHttpClient client33 = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType33 = MediaType.parse("text/plain");
            RequestBody body33 = RequestBody.create(mediaType33, "");
            Request request33 = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/project/" + projectId + "/taskflow/search")
//                .method("GET", body33)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .build();
            int code33 = 0;
            JSONArray result33 = null;
            try {
                Response response = client33.newCall(request33).execute();
                String bodyStr = response.body().string();
                JSONObject jsonObject33 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject33);
                code33 = jsonObject33.getInteger("code");

                result33 = jsonObject33.getJSONArray("result");
            } catch (IOException e) {
                e.printStackTrace();
            }

            String gongzuoliuID = null;
            for (int i = 0; i < result33.size(); i++) {
                String name = result33.getJSONObject(i).getString("name");
                System.out.println("工作流名称：" + name);
                if (name.equals("关键任务变更")){
                    String id = result33.getJSONObject(i).getString("id");
                    System.out.println("工作流id：" + id);
                    gongzuoliuID = id;
                }
            }

//            String gongzuoliuID = result33.getJSONObject(0).getString("id");
            System.out.println("查询【项目的工作流】接口返回的状态码（200成功，400异常）：" + code33);
            System.out.println("项目工作流ID（任务变更工作流）：" + gongzuoliuID);

            //根据项目默认工作流ID获取项目中任务的各个状态ID
            OkHttpClient client44 = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType44 = MediaType.parse("text/plain");
            RequestBody body44 = RequestBody.create(mediaType44, "");
            Request request44 = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/project/" + projectId + "/taskflowstatus/search?tfIds=" + gongzuoliuID)
//                .method("GET", body44)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .build();
            int code44 = 0;
            JSONArray result44 = null;
            try {
                Response response44 = client44.newCall(request44).execute();
                String bodyStr = response44.body().string();
                JSONObject jsonObject44 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject44);
                code44 = jsonObject44.getInteger("code");

                result44 = jsonObject44.getJSONArray("result");
            } catch (IOException e) {
                e.printStackTrace();
            }

//        String renwuID = result44.getJSONObject(2).getString("id");

            String zhuangtai = null;
            if (shenheyijian.equals("同意")){
                zhuangtai = "审核通过";
            }else {
                zhuangtai = "审核不通过";
            }

            String renwuID = null;
            for (int i = 0; i < result44.size(); i++) {
                String name = result44.getJSONObject(i).getString("name");
                System.out.println("任务状态名称：" + name);
                if (name.equals(zhuangtai)) {
                    String id = result44.getJSONObject(i).getString("id");
                    renwuID = id;
                }
            }

//        result44.stream().filter()

            System.out.println("--------------------------");
            System.out.println("查询【项目关键任务变更工作流ID】接口返回的状态码（200成功，400异常）：" + code44);
            System.out.println("项目工作流-->任务完成ID：" + renwuID);

            //TB任务状态id
            // 未处理：64afa1090c87b662c9b28cda
            // 已完成：64afa1090c87b662c9b28cdc

            //柏TBuserid：61c68fc2947cb43084ec0113
            //牧之TBuserid：5f3a250040ce230ba377bf3c
            //更新TB任务状态
            log.info("开始执行更新TB任务状态操作...");

            Map map = new HashMap();
            map.put("taskflowstatusId", renwuID);
            String mapJson = JSON.toJSONString(map);
            System.out.println("查询条件mapJson：" + mapJson);

            OkHttpClient client = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType = MediaType.parse("application/json");
            RequestBody body = RequestBody.create(mediaType, mapJson);
            Request request = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/task/" + task + "/taskflowstatus")
                    .method("PUT", body)
                    //TODO 此处需要替换成项目创建者？？？【已和客户确认】
                    .addHeader("x-operator-id", id1)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .addHeader("Content-Type", "application/json")
                    .build();

            int code11 = 0;
            try {
                Response response = client.newCall(request).execute();
                String bodyStr = response.body().string();
                JSONObject jsonObject11 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject11);
                code11 = jsonObject11.getInteger("code");
            } catch (IOException e) {
                e.printStackTrace();
            }
            System.out.println("更新【TB任务状态】接口返回的状态码（200成功，400异常）：" + code11);
        }

        return "TB任务【关键任务变更】状态更新完成！";
    }


    //获取钉钉OA审批信息（额外增加1：会议决议完成情况审核）
    @Override
    public String getDingOAbiangeng3(String processInstances) {

//        processInstances = "kYMesPtPS1CkGVVDuePG3Q01951689761375";

        log.info("钉钉连接器传参审批实例ID：" + processInstances);

        //根据审批实力id获取审批单详情，并获取评论添加的tb任务详情，最后取值任务id
        //一、获取【任务完成审批】详情
        JSONObject jsonObject = null;
        try {
            com.aliyun.dingtalkworkflow_1_0.Client client = createClient();
            com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceHeaders getProcessInstanceHeaders = new com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceHeaders();
            getProcessInstanceHeaders.xAcsDingtalkAccessToken = accessTokenService.getAccessToken();
            com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceRequest getProcessInstanceRequest = new com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceRequest()
                    //审批实例id
                    .setProcessInstanceId(processInstances);

            GetProcessInstanceResponse processInstanceWithOptions = client.getProcessInstanceWithOptions(getProcessInstanceRequest, getProcessInstanceHeaders, new RuntimeOptions());
//            System.out.println(JSON.toJSONString(processInstanceWithOptions.getBody()));
            String s = JSON.toJSONString(processInstanceWithOptions.getBody());
            jsonObject = JSON.parseObject(s);
            System.out.println("审批详情接口object对象：" + jsonObject);
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                // err 中含有 code 和 message 属性，可帮助开发定位问题
            }
        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                // err 中含有 code 和 message 属性，可帮助开发定位问题
            }
        }
        JSONObject result = jsonObject.getJSONObject("result");
        System.out.println("result：" + result);
        JSONArray formComponentValues = result.getJSONArray("formComponentValues");
        System.out.println("formComponentValues：" + formComponentValues);

        JSONArray operationRecords = result.getJSONArray("operationRecords");
        JSONObject jsonObject1 = operationRecords.getJSONObject(1);
        String remark = jsonObject1.getString("remark");
        System.out.println("评论地址：" + remark);

        String substring = remark.substring(remark.indexOf("(") + 1, remark.indexOf(")"));
        System.out.println("截取出来的url：" + substring);


        //审批状态
        //NEW：新创建
        //RUNNING：审批中
        //TERMINATED：被终止
        //COMPLETED：完成
        //CANCELED：取消
        String status = result.getString("status");
        System.out.println("当前审批单状态：" + status);

        String shenpijieguo = null;
        try {
            shenpijieguo = result.getString("result");
            System.out.println("当前审批单结果：" + shenpijieguo);
        }catch (Exception e){
            e.printStackTrace();
        }
        if (status.equals("COMPLETED") && shenpijieguo.equals("agree")) {
            log.info("审批通过，开始执行更新TB任务状态动作～");

            String url = substring;
            for (int i = 0; i < 3; i++) {


                String decodedUrl = null;
                try {
                    decodedUrl = URLDecoder.decode(url, "UTF-8");
                } catch (Exception e) {
                    e.printStackTrace();
                }
                System.out.println("第" + i + "次解码后：" + decodedUrl);

                url = decodedUrl;
            }
            System.out.println("3次最终解码后的url：" + url);

            //截取指定的task任务编号
//            String task = url.substring(url.indexOf("tasks/") + 6, url.indexOf("&productCode"));
            //TODO 特殊处理
            String task = url.substring(url.indexOf("tasks/") + 6, url.indexOf("?isDingtalkHeaderHidden"));
            //?isDingtalkHeaderHidden=true
            System.out.println("截取的task任务编号：" + task);

            //查询任务详情，获取该任务的项目ID
            OkHttpClient client22 = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType22 = MediaType.parse("text/plain");
            RequestBody body22 = RequestBody.create(mediaType22, "");
            Request request22 = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/task/query?taskId=" + task)
//                .method("GET", body)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .build();

            int code22 = 0;
            JSONArray result22 = null;
            try {
                Response response = client22.newCall(request22).execute();
                String bodyStr = response.body().string();
                JSONObject jsonObject22 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject22);
                code22 = jsonObject22.getInteger("code");

                result22 = jsonObject22.getJSONArray("result");
            } catch (IOException e) {
                e.printStackTrace();
            }
            //项目id
            String projectId = result22.getJSONObject(0).getString("projectId");
            //创建人id
            String id1 = result22.getJSONObject(0).getString("creatorId");
            System.out.println("查询【任务详情】接口返回的状态码（200成功，400异常）：" + code22);
            System.out.println("项目ID：" + projectId);
            System.out.println("创建人id：" + id1);

            //查询项目的工作流（获取默认工作流ID）
            OkHttpClient client33 = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType33 = MediaType.parse("text/plain");
            RequestBody body33 = RequestBody.create(mediaType33, "");
            Request request33 = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/project/" + projectId + "/taskflow/search")
//                .method("GET", body33)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .build();
            int code33 = 0;
            JSONArray result33 = null;
            try {
                Response response = client33.newCall(request33).execute();
                String bodyStr = response.body().string();
                JSONObject jsonObject33 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject33);
                code33 = jsonObject33.getInteger("code");

                result33 = jsonObject33.getJSONArray("result");
            } catch (IOException e) {
                e.printStackTrace();
            }

            String gongzuoliuID = null;
            for (int i = 0; i < result33.size(); i++) {
                String name = result33.getJSONObject(i).getString("name");
                System.out.println("工作流模板名称：" + name);

                if (name.equals("默认工作流")){
                    String id = result33.getJSONObject(i).getString("id");
                    System.out.println("工作流模板【默认工作流】id：" + id);
                    gongzuoliuID = id;
                }
            }

//            String gongzuoliuID = result33.getJSONObject(0).getString("id");
            System.out.println("查询【项目的工作流】接口返回的状态码（200成功，400异常）：" + code33);
            System.out.println("项目工作流ID（默认工作流）：" + gongzuoliuID);

            //根据项目默认工作流ID获取项目中任务的各个状态ID
            OkHttpClient client44 = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType44 = MediaType.parse("text/plain");
            RequestBody body44 = RequestBody.create(mediaType44, "");
            Request request44 = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/project/" + projectId + "/taskflowstatus/search?tfIds=" + gongzuoliuID)
//                .method("GET", body44)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .build();
            int code44 = 0;
            JSONArray result44 = null;
            try {
                Response response44 = client44.newCall(request44).execute();
                String bodyStr = response44.body().string();
                JSONObject jsonObject44 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject44);
                code44 = jsonObject44.getInteger("code");

                result44 = jsonObject44.getJSONArray("result");
            } catch (IOException e) {
                e.printStackTrace();
            }

//        String renwuID = result44.getJSONObject(2).getString("id");

            String renwuID = null;
            for (int i = 0; i < result44.size(); i++) {
                String name = result44.getJSONObject(i).getString("name");
                System.out.println("任务状态名称：" + name);
                if (name.equals("已完成")) {
                    String id = result44.getJSONObject(i).getString("id");
                    renwuID = id;
                }
            }

//        result44.stream().filter()

            System.out.println("--------------------------");
            System.out.println("查询【项目默认工作流ID】接口返回的状态码（200成功，400异常）：" + code44);
            System.out.println("项目工作流-->任务完成ID：" + renwuID);

            //TB任务状态id
            // 未处理：64afa1090c87b662c9b28cda
            // 已完成：64afa1090c87b662c9b28cdc

            //柏TBuserid：61c68fc2947cb43084ec0113
            //牧之TBuserid：5f3a250040ce230ba377bf3c
            //更新TB任务状态
            log.info("开始执行更新TB任务状态操作...");

            Map map = new HashMap();
            map.put("taskflowstatusId", renwuID);
            String mapJson = JSON.toJSONString(map);
            System.out.println("查询条件mapJson：" + mapJson);

            OkHttpClient client = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType = MediaType.parse("application/json");
            RequestBody body = RequestBody.create(mediaType, mapJson);
            Request request = new Request.Builder()
                    .url("https://open.teambition.com/api/v3/task/" + task + "/taskflowstatus")
                    .method("PUT", body)
                    //TODO 此处需要替换成项目创建者？？？【已和客户确认】
                    .addHeader("x-operator-id", id1)
                    .addHeader("Authorization", "Bearer " + accessTokenService.getAppToken())
                    .addHeader("X-Tenant-Id", tbOrganizationId)
                    .addHeader("X-Tenant-Type", "organization")
                    .addHeader("Content-Type", "application/json")
                    .build();

            int code11 = 0;
            try {
                Response response = client.newCall(request).execute();
                String bodyStr = response.body().string();
                JSONObject jsonObject11 = JSON.parseObject(bodyStr);
                System.out.println("返回体body：" + jsonObject11);
                code11 = jsonObject11.getInteger("code");
            } catch (IOException e) {
                e.printStackTrace();
            }
            System.out.println("更新【TB任务状态】接口返回的状态码（200成功，400异常）：" + code11);
        }

        return "TB【额外增加1：会议决议完成情况审核】任务状态更新完成！";
    }
}
