package com.muzhi.tianhe.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.muzhi.tianhe.service.ThTbService;
import com.muzhi.tianhe.util.PublicUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class ThTbServiceImpl implements ThTbService {

    @Autowired
    private TbApiService tbApiService;


    @Override
    public void syncTaskState(String taskId) {
        JSONObject task=tbApiService.getTasksInfo(taskId).getJSONObject(0);
        String tfsId=task.getString("tfsId");// 状态ID
        String projectId=task.getString("projectId");
        JSONObject tfs=tbApiService.getProjectStateId(projectId,"","",tfsId).getJSONObject(0);
        if(task.getJSONArray("ancestorIds").size()<3){
            log.info("非指标任务 不处理");
            return;
        }
        updateTaskState(task);
    }

    public void updateTaskState(JSONObject task){
        String projectId=task.getString("projectId");// 项目ID
        // 上级任务信息
        String parentTaskId=task.getString("parentTaskId");// 上级任务ID
        JSONObject parentTask=tbApiService.getTasksInfo(parentTaskId).getJSONObject(0);
        JSONObject parentTaskTfs=tbApiService.getProjectStateId(projectId,"","",parentTask.getString("tfsId")).getJSONObject(0);
        String parentTaskTfsName=parentTaskTfs.getString("name");
        String type=getType(task.getJSONArray("ancestorIds").size());
        String newTfsName;
        // 循环获取其他同级任务状态，判断是否有更高级别的任务状态
        // 同级红灯状态ID
        JSONObject tf=tbApiService.getProjectTfid(projectId,getType(task.getJSONArray("ancestorIds").size()+1)).getJSONObject(0);
        JSONObject redTfs=tbApiService.getProjectStateId(projectId,"红灯",tf.getString("id"),"").getJSONObject(0);
        String redId=redTfs.getString("id");
        // 同级黄状态ID
        JSONObject yellowTfs=tbApiService.getProjectStateId(projectId,"黄灯",tf.getString("id"),"").getJSONObject(0);
        String yellowId=yellowTfs.getString("id");
        JSONArray array=tbApiService.getTasksByParent(parentTaskId);
        String color="绿灯";
        int redNum=0,yellowNum=0;
        for (int i = 0; i < array.size(); i++) {
            JSONObject taskT=array.getJSONObject(i);
            if(redId.equals(taskT.getString("tfsId"))){
                redNum++;
            }else if(yellowId.equals(taskT.getString("tfsId"))){
                yellowNum++;
            }
        }
        if(yellowNum>=2){
            color="黄灯";
        }
        if(redNum>=2){
            color="红灯";
        }
        if(!color.equals(parentTaskTfsName)){
            newTfsName=color;
        }else{
            return;
        }
        update(projectId,parentTaskId,newTfsName,type);
        if(!PublicUtil.isNull(parentTask.getString("parentTaskId"))){
            updateTaskState(parentTask);
        }
    }

    public String getType(int size){
        switch (size){
            case 1:
                return"一级任务工作流";
            case 2:
                return "二级任务工作流";
            case 3:
                return "三级任务工作流";
            case 4:
                return "年度指标工作流";
            default:return "";
        }
    }

    public void update(String projectId,String taskId,String tfsName,String type){
        log.info("任务[{}]状态修改为[{}]",taskId,tfsName);
        JSONObject tf=tbApiService.getProjectTfid(projectId,type).getJSONObject(0);
        JSONObject tfsnext=tbApiService.getProjectStateId(projectId,tfsName,tf.getString("id"),"").getJSONObject(0);
        tbApiService.updateTaskStatus(taskId,tfsnext.getString("id"));
    }
}
